/*
 * testfamcoll.c: test program for the famcoll module.
 *
 * (C) Duncan C. White, May 2017
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdbool.h>
#include <assert.h>

#include "set.h"
#include "hash.h"

#include "famcoll.h"


//#define TESTSET


static famcoll f;


#if 1
static char *bool2str[] = { "false", "true" };

static void testbool( bool v, bool expected, char *msg )
{
	if( v != expected )
	{
		printf( "T %s: should be %s, was %s: bad\n",
			msg, bool2str[expected], bool2str[v] );
	} else
	{
		printf( "T %s: is %s: ok\n", msg, bool2str[v] );
	}
}
#endif


static void testint( int v, int expected, char *msg )
{
	if( v != expected )
	{
		printf( "T %s: should be %d, was %d: bad\n",
			msg, expected, v );
	} else
	{
		printf( "T %s: is %d: ok\n", msg, v );
	}
}


/*
 * testcontains( f, parent, csvchildren );
 *	test that famcoll f contains an entry for parent, containing
 *	precisely the comma-separated csvchildren
 *	(and no other children).  assumes that child names contain
 *	no commas themselves:-)
 */
void testcontains( famcoll f, char *parent, char *csvchildren )
{
	set s = famcoll_children( f, parent );

	// we need to modify the string, so make a mutable copy..
	csvchildren = strdup( csvchildren );

	char msg[1024];
	sprintf( msg, "f(%s) contains %s", parent, csvchildren );

	// foreach csv value in csvchildren..
	int nfound = 0;
	int nincsv = 0;
	char *element = csvchildren;
	for(;;)
	{
		char *comma=strchr( element, ',' );
		if( comma != NULL )
		{
			// found the first comma after element
			*comma = '\0';
		}
		//printf( "debug: csvForeach: found value %s\n", element );
		nincsv++;

		// check whether element is in the set (as it should be!)
		if( set_in( s, element ) )
		{
			// good: expected element to be in, and it is.
			printf( "T %s: %s is in set: ok\n", msg, element );
			nfound++;
		} else
		{
			printf( "T %s: %s should be in set but isn't: bad\n",
				msg, element );
		}

	if( comma == NULL ) break;

		// move start to one beyond where comma was..
		element = comma+1;
	}
	// don't forget to free the copy..
	free( csvchildren );

	sprintf( msg, "f(%s) has %d children(s)", parent, nincsv );
	testint( nfound, nincsv, msg );
}

int main( int argc, char **argv )
{
	if( argc > 1 )
	{
		malloc(strlen(argv[1]));	/* test memory leaks */
	}

	#ifdef TESTSET
	set s = set_create( NULL );
	set_add( s, "one" );
	printf( "set s = " );
	set_dump( stdout, s );
	printf( "\n" );
	testbool( setIn( s, "one" ), true, "one in s" );
	testint( set_nmembers(s), 1, "s has 1 entry" );
	#endif

	f = famcoll_create();
	famcoll_addchild( f, "one", "a" );
	printf( "added <a> to <one> in f\n" );
	testint( famcoll_nfamilies(f), 1, "f has 1 family" );

	set s2 = famcoll_children( f, "one" );
	testint( set_nmembers(s2), 1, "f[one] has 1 child" );

	testbool( set_in(s2, "a" ), true, "a in f[one]" );
	testbool( set_in(s2, "b" ), false, "b in f[one]" );

	printf( "initial families:\n" );
	famcoll_dump( stdout, f );

	testcontains( f, "one", "a" );	// test that "one" contains only "a"

	famcoll_addchild( f, "one", "b" );
	printf( "added <b> to <one> in f\n" );
	testcontains( f, "one", "a,b" );	// should contain a and b

	famcoll_addchild( f, "one", "c" );
	printf( "added <c> to <one> in f\n" );
	testcontains( f, "one", "a,b,c" );	// should contain a,b and c

	famcoll_addchild( f, "two", "a" );
	printf( "added <a> to <two> in f\n" );
	testcontains( f, "one", "a,b,c" );
	testcontains( f, "two", "a" );

	famcoll_addchild( f, "three", "z" );
	printf( "added <z> to <three> in f\n" );
	testcontains( f, "one", "a,b,c" );
	testcontains( f, "two", "a" );
	testcontains( f, "three", "z" );

	famcoll_addchild( f, "two", "d" );
	printf( "added <d> to <two> in f\n" );
	testcontains( f, "one", "a,b,c" );
	testcontains( f, "two", "d,a" );	// order irrelevent

	printf( "final families:\n" );
	famcoll_dump( stdout, f );

	famcoll_free( f );

	return 0;
}
